@@ -35,3 +35,20 @@ devnet_banks: list[Bank] = [
         mint=PublicKey("3BZPwbcqB5kKScF3TEXxwNfx5ipV13kbRVDvfVp5c6fv"),
     ),
 ]
+
+mainnet_banks: list[Bank] = [
+    Bank(
+        symbol="USDC",
+        bank_index=0,
+        oracle=PublicKey(0),
+        oracle_source=OracleSource.QUOTE_ASSET,
+        mint=PublicKey("8zGuJQqwhZafTah7Uc7Z4tXRnguqkn5KLFAP8oV6PHe2"),
+    ),
+    Bank(
+        symbol="SOL",
+        bank_index=1,
+        oracle=PublicKey("H6ARHf6YXhGYeQfUzQNGk6rDNnLBQKrenN712K4AQJEG"),
+        oracle_source=OracleSource.PYTH,
+        mint=PublicKey("So11111111111111111111111111111111111111112"),
+    ),
+]
