@@ -4,43 +4,36 @@ from borsh_construct.enum import _rust_enum
 from sumtypes import constructor
 from typing import Optional
 
-
 @_rust_enum
 class SwapDirection:
     ADD = constructor()
     REMOVE = constructor()
-
-
+ 
 @_rust_enum
 class PositionDirection:
     LONG = constructor()
     SHORT = constructor()
-
-
+ 
 @_rust_enum
 class SpotFulfillmentType:
     SERUM_V3 = constructor()
     NONE = constructor()
-
-
+ 
 @_rust_enum
 class TwapPeriod:
     FUNDING_PERIOD = constructor()
     FIVE_MIN = constructor()
-
-
+ 
 @_rust_enum
 class LiquidationMultiplierType:
     DISCOUNT = constructor()
     PREMIUM = constructor()
-
-
+ 
 @_rust_enum
 class MarginRequirementType:
     INITIAL = constructor()
     MAINTENANCE = constructor()
-
-
+ 
 @_rust_enum
 class OracleValidity:
     INVALID = constructor()
@@ -50,8 +43,7 @@ class OracleValidity:
     INSUFFICIENT_DATA_POINTS = constructor()
     STALE_FOR_A_M_M = constructor()
     VALID = constructor()
-
-
+ 
 @_rust_enum
 class DriftAction:
     UPDATE_FUNDING = constructor()
@@ -63,8 +55,7 @@ class DriftAction:
     MARGIN_CALC = constructor()
     UPDATE_TWAP = constructor()
     UPDATE_A_M_M_CURVE = constructor()
-
-
+ 
 @_rust_enum
 class PositionUpdateType:
     OPEN = constructor()
@@ -72,20 +63,17 @@ class PositionUpdateType:
     REDUCE = constructor()
     CLOSE = constructor()
     FLIP = constructor()
-
-
+ 
 @_rust_enum
 class DepositExplanation:
     NONE = constructor()
     TRANSFER = constructor()
-
-
+ 
 @_rust_enum
 class DepositDirection:
     DEPOSIT = constructor()
     WITHDRAW = constructor()
-
-
+ 
 @_rust_enum
 class OrderAction:
     PLACE = constructor()
@@ -93,8 +81,7 @@ class OrderAction:
     FILL = constructor()
     TRIGGER = constructor()
     EXPIRE = constructor()
-
-
+ 
 @_rust_enum
 class OrderActionExplanation:
     NONE = constructor()
@@ -111,15 +98,14 @@ class OrderActionExplanation:
     RISKING_INCREASING_ORDER = constructor()
     REDUCE_ONLY_ORDER_INCREASED_POSITION = constructor()
     ORDER_FILL_WITH_SERUM = constructor()
-
-
+    NO_BORROW_LIQUIDITY = constructor()
+ 
 @_rust_enum
 class LPAction:
     ADD_LIQUIDITY = constructor()
     REMOVE_LIQUIDITY = constructor()
     SETTLE_LIQUIDITY = constructor()
-
-
+ 
 @_rust_enum
 class LiquidationType:
     LIQUIDATE_PERP = constructor()
@@ -128,41 +114,35 @@ class LiquidationType:
     LIQUIDATE_PERP_PNL_FOR_DEPOSIT = constructor()
     PERP_BANKRUPTCY = constructor()
     SPOT_BANKRUPTCY = constructor()
-
-
+ 
 @_rust_enum
 class SettlePnlExplanation:
     NONE = constructor()
     EXPIRED_POSITION = constructor()
-
-
+ 
 @_rust_enum
 class StakeAction:
     STAKE = constructor()
     UNSTAKE_REQUEST = constructor()
     UNSTAKE_CANCEL_REQUEST = constructor()
     UNSTAKE = constructor()
-
-
+ 
 @_rust_enum
 class PerpFulfillmentMethod:
     A_M_M = constructor()
     MATCH = constructor()
-
-
+ 
 @_rust_enum
 class SpotFulfillmentMethod:
     SERUM_V3 = constructor()
     MATCH = constructor()
-
-
+ 
 @_rust_enum
 class OracleSource:
     PYTH = constructor()
     SWITCHBOARD = constructor()
     QUOTE_ASSET = constructor()
-
-
+ 
 @_rust_enum
 class MarketStatus:
     INITIALIZED = constructor()
@@ -174,14 +154,12 @@ class MarketStatus:
     REDUCE_ONLY = constructor()
     SETTLEMENT = constructor()
     DELISTED = constructor()
-
-
+ 
 @_rust_enum
 class ContractType:
     PERPETUAL = constructor()
     FUTURE = constructor()
-
-
+ 
 @_rust_enum
 class ContractTier:
     A = constructor()
@@ -189,20 +167,17 @@ class ContractTier:
     C = constructor()
     SPECULATIVE = constructor()
     ISOLATED = constructor()
-
-
+ 
 @_rust_enum
 class SpotBalanceType:
     DEPOSIT = constructor()
     BORROW = constructor()
-
-
+ 
 @_rust_enum
-class SpotFulfillmentStatus:
+class SpotFulfillmentConfigStatus:
     ENABLED = constructor()
     DISABLED = constructor()
-
-
+ 
 @_rust_enum
 class AssetTier:
     COLLATERAL = constructor()
@@ -210,8 +185,7 @@ class AssetTier:
     CROSS = constructor()
     ISOLATED = constructor()
     UNLISTED = constructor()
-
-
+ 
 @_rust_enum
 class ExchangeStatus:
     ACTIVE = constructor()
@@ -221,51 +195,45 @@ class ExchangeStatus:
     LIQ_PAUSED = constructor()
     WITHDRAW_PAUSED = constructor()
     PAUSED = constructor()
-
-
+ 
 @_rust_enum
 class UserStatus:
     ACTIVE = constructor()
     BEING_LIQUIDATED = constructor()
     BANKRUPT = constructor()
-
-
+ 
 @_rust_enum
 class AssetType:
     BASE = constructor()
     QUOTE = constructor()
-
-
+ 
 @_rust_enum
 class OrderStatus:
     INIT = constructor()
     OPEN = constructor()
     FILLED = constructor()
     CANCELED = constructor()
-
-
+ 
 @_rust_enum
 class OrderType:
     MARKET = constructor()
     LIMIT = constructor()
     TRIGGER_MARKET = constructor()
     TRIGGER_LIMIT = constructor()
-
-
+    ORACLE = constructor()
+ 
 @_rust_enum
 class OrderTriggerCondition:
     ABOVE = constructor()
     BELOW = constructor()
     TRIGGERED_ABOVE = constructor()
     TRIGGERED_BELOW = constructor()
-
-
+ 
 @_rust_enum
 class MarketType:
     SPOT = constructor()
     PERP = constructor()
-
-
+ 
 @dataclass
 class OrderParams:
     order_type: OrderType
@@ -285,8 +253,7 @@ class OrderParams:
     auction_duration: Optional[int]
     auction_start_price: Optional[int]
     auction_end_price: Optional[int]
-
-
+ 
 @dataclass
 class HistoricalOracleData:
     last_oracle_price: int
@@ -295,15 +262,13 @@ class HistoricalOracleData:
     last_oracle_price_twap: int
     last_oracle_price_twap5min: int
     last_oracle_price_twap_ts: int
-
-
+ 
 @dataclass
 class PoolBalance:
     scaled_balance: int
     market_index: int
     padding: list[int]
-
-
+ 
 @dataclass
 class AMM:
     oracle: PublicKey
@@ -382,28 +347,24 @@ class AMM:
     oracle_source: OracleSource
     last_oracle_valid: bool
     padding: list[int]
-
-
+ 
 @dataclass
 class PriceDivergenceGuardRails:
     mark_oracle_divergence_numerator: int
     mark_oracle_divergence_denominator: int
-
-
+ 
 @dataclass
 class ValidityGuardRails:
     slots_before_stale_for_amm: int
     slots_before_stale_for_margin: int
     confidence_interval_max_size: int
     too_volatile_ratio: int
-
-
+ 
 @dataclass
 class OracleGuardRails:
     price_divergence: PriceDivergenceGuardRails
     validity: ValidityGuardRails
-
-
+ 
 @dataclass
 class FeeTier:
     fee_numerator: int
@@ -414,23 +375,20 @@ class FeeTier:
     referrer_reward_denominator: int
     referee_fee_numerator: int
     referee_fee_denominator: int
-
-
+ 
 @dataclass
 class OrderFillerRewardStructure:
     reward_numerator: int
     reward_denominator: int
     time_based_reward_lower_bound: int
-
-
+ 
 @dataclass
 class FeeStructure:
     fee_tiers: list[FeeTier]
     filler_reward_structure: OrderFillerRewardStructure
     referrer_reward_epoch_upper_bound: int
     flat_filler_fee: int
-
-
+ 
 @dataclass
 class SpotPosition:
     scaled_balance: int
@@ -441,8 +399,7 @@ class SpotPosition:
     balance_type: SpotBalanceType
     open_orders: int
     padding: list[int]
-
-
+ 
 @dataclass
 class Order:
     slot: int
@@ -469,8 +426,7 @@ class Order:
     trigger_condition: OrderTriggerCondition
     auction_duration: int
     padding: list[int]
-
-
+ 
 @dataclass
 class InsuranceClaim:
     revenue_withdraw_since_last_settle: int
@@ -478,8 +434,7 @@ class InsuranceClaim:
     quote_max_insurance: int
     quote_settled_insurance: int
     last_revenue_withdraw_ts: int
-
-
+ 
 @dataclass
 class PerpMarket:
     pubkey: PublicKey
@@ -508,8 +463,7 @@ class PerpMarket:
     contract_type: ContractType
     contract_tier: ContractTier
     padding: list[int]
-
-
+ 
 @dataclass
 class HistoricalIndexData:
     last_index_bid_price: int
@@ -517,8 +471,7 @@ class HistoricalIndexData:
     last_index_price_twap: int
     last_index_price_twap5min: int
     last_index_price_twap_ts: int
-
-
+ 
 @dataclass
 class InsuranceFund:
     vault: PublicKey
@@ -530,8 +483,7 @@ class InsuranceFund:
     revenue_settle_period: int
     total_factor: int
     user_factor: int
-
-
+ 
 @dataclass
 class SpotMarket:
     pubkey: PublicKey
@@ -582,8 +534,7 @@ class SpotMarket:
     status: MarketStatus
     asset_tier: AssetTier
     padding: list[int]
-
-
+ 
 @dataclass
 class SerumV3FulfillmentConfig:
     pubkey: PublicKey
@@ -599,10 +550,9 @@ class SerumV3FulfillmentConfig:
     serum_signer_nonce: int
     market_index: int
     fulfillment_type: SpotFulfillmentType
-    status: SpotFulfillmentStatus
+    status: SpotFulfillmentConfigStatus
     padding: list[int]
-
-
+ 
 @dataclass
 class State:
     admin: PublicKey
@@ -625,9 +575,10 @@ class State:
     default_market_order_time_in_force: int
     default_spot_auction_duration: int
     exchange_status: ExchangeStatus
+    liquidation_duration: int
+    initial_pct_to_liquidate: int
     padding: list[int]
-
-
+ 
 @dataclass
 class PerpPosition:
     last_cumulative_funding_rate: int
@@ -639,14 +590,13 @@ class PerpPosition:
     open_asks: int
     settled_pnl: int
     lp_shares: int
-    last_net_base_asset_amount_per_lp: int
-    last_net_quote_asset_amount_per_lp: int
+    last_base_asset_amount_per_lp: int
+    last_quote_asset_amount_per_lp: int
     remainder_base_asset_amount: int
     market_index: int
     open_orders: int
     padding: list[int]
-
-
+ 
 @dataclass
 class User:
     authority: PublicKey
@@ -663,7 +613,7 @@ class User:
     cumulative_spot_fees: int
     cumulative_perp_funding: int
     liquidation_margin_freed: int
-    liquidation_start_ts: int
+    liquidation_start_slot: int
     next_order_id: int
     max_margin_ratio: int
     next_liquidation_id: int
@@ -671,8 +621,7 @@ class User:
     status: UserStatus
     is_margin_trading_enabled: bool
     padding: list[int]
-
-
+ 
 @dataclass
 class UserFees:
     total_fee_paid: int
@@ -681,8 +630,7 @@ class UserFees:
     total_referee_discount: int
     total_referrer_reward: int
     current_epoch_referrer_reward: int
-
-
+ 
 @dataclass
 class UserStats:
     authority: PublicKey
@@ -700,8 +648,7 @@ class UserStats:
     number_of_sub_accounts_created: int
     is_referrer: bool
     padding: list[int]
-
-
+ 
 @dataclass
 class LiquidatePerpRecord:
     market_index: int
@@ -714,8 +661,7 @@ class LiquidatePerpRecord:
     liquidator_order_id: int
     liquidator_fee: int
     if_fee: int
-
-
+ 
 @dataclass
 class LiquidateSpotRecord:
     asset_market_index: int
@@ -725,8 +671,7 @@ class LiquidateSpotRecord:
     liability_price: int
     liability_transfer: int
     if_fee: int
-
-
+ 
 @dataclass
 class LiquidateBorrowForPerpPnlRecord:
     perp_market_index: int
@@ -735,8 +680,7 @@ class LiquidateBorrowForPerpPnlRecord:
     liability_market_index: int
     liability_price: int
     liability_transfer: int
-
-
+ 
 @dataclass
 class LiquidatePerpPnlForDepositRecord:
     perp_market_index: int
@@ -745,8 +689,7 @@ class LiquidatePerpPnlForDepositRecord:
     asset_market_index: int
     asset_price: int
     asset_transfer: int
-
-
+ 
 @dataclass
 class PerpBankruptcyRecord:
     market_index: int
@@ -755,16 +698,14 @@ class PerpBankruptcyRecord:
     clawback_user: Optional[PublicKey]
     clawback_user_payment: Optional[int]
     cumulative_funding_rate_delta: int
-
-
+ 
 @dataclass
 class SpotBankruptcyRecord:
     market_index: int
     borrow_amount: int
     if_payment: int
     cumulative_deposit_interest_delta: int
-
-
+ 
 @dataclass
 class InsuranceFundStake:
     authority: PublicKey
@@ -777,3 +718,4 @@ class InsuranceFundStake:
     cost_basis: int
     market_index: int
     padding: list[int]
+ 
